from tests.test_actions import *
from ltk.actions import Action
import os

def test_add_db():
    # check document added to db
    create_config()
    action = Action(os.getcwd())
    file_name = 'sample.txt'
    text_file_path = create_txt_file(file_name)
    action.add_action(None, [file_name])
    # check that document added in db
    assert action.doc_manager.get_doc_by_prop('name', file_name)
    action.delete_action(file_name)
    action.clean_action(True, False, None)
    # os.remove(text_file_path)
    cleanup()

def test_add_remote():
    # check that document added to Lingotek
    create_config()
    action = Action(os.getcwd())
    file_name = 'sample.txt'
    text_file_path = create_txt_file(file_name)
    action.add_action(None, [file_name])
    doc_id = action.doc_manager.get_doc_ids()[0]
    # response = action.api.get_document(doc_id)
    # assert response.status_code == 200
    assert poll_doc(action, doc_id)
    action.delete_action(file_name)
    action.clean_action(True, False, None)
    # os.remove(text_file_path)
    cleanup()

def test_add_pattern_db():
    # test that adding with a pattern gets all expected matches in local db
    create_config()
    action = Action(os.getcwd())
    files = ['sample.txt', 'sample1.txt', 'sample2.txt']
    file_paths = []
    for fn in files:
        file_paths.append(create_txt_file(fn))
    action.add_action(None, ['sample*.txt'])
    for fn in files:
        assert action.doc_manager.get_doc_by_prop('name', fn)
    # for file_path in file_paths:
    #     os.remove(file_path)
    for fn in files:
        action.delete_action(fn)
    action.clean_action(True, False, None)
    cleanup()

def test_add_pattern_remote():
    # test that adding with a pattern gets all expected matches in Lingotek
    create_config()
    action = Action(os.getcwd())
    files = ['sample.txt', 'sample1.txt', 'sample2.txt']
    file_paths = []
    for fn in files:
        file_paths.append(create_txt_file(fn))
    action.add_action(None, ['sample*.txt'])
    doc_ids = action.doc_manager.get_doc_ids()
    for doc_id in doc_ids:
        assert poll_doc(action, doc_id)
    # for file_path in file_paths:
    #     os.remove(file_path)
    for fn in files:
        action.delete_action(fn)
    action.clean_action(True, False, None)
    cleanup()

# todo test all those other args



from tests.test_actions import *
from ltk import actions
import unittest

class TestConfig(unittest.TestCase):
    def setUp(self):
        create_config()
        self.action = actions.Action(os.getcwd())

    def tearDown(self):
        cleanup()

    def test_config(self):
        from io import BytesIO
        import sys

        try:
            out = BytesIO()
            sys.stdout = out
            self.action.config_action(None, None, None, None, [])
            info = out.getvalue()
            assert info.startswith('host: https://cms.lingotek.com')
        finally:
            sys.stdout = sys.__stdout__

    def test_change_locale(self):
        self.action.config_action('de_DE', None, None, None, [])
        assert self.action.locale == 'de_DE'

    def test_change_workflow(self):
        new_workflow = '6ff1b470-33fd-11e2-81c1-0800200c9a66'
        self.action.config_action(None, new_workflow, None, None, [])
        assert self.action.workflow_id == new_workflow

    def test_add_download_folder(self):
        download_folder = 'downloaded'
        self.action.config_action(None, None, download_folder, None, [])
        assert self.action.download_dir == os.path.join(self.action.path, download_folder)

    def test_add_upload_folder(self):
        watch_folder = 'watching'
        self.action.config_action(None, None, None, watch_folder, [])
        assert self.action.watch_dir == os.path.join(self.action.path, watch_folder)



from ltk.actions import *


class ImportAction(Action):
    def __init__(self, path):
        Action.__init__(self, path)

    def import_action(self, import_all, force, path):
        response = self.api.list_documents(self.project_id)
        tms_doc_info = {}
        if response.status_code == 200:
            tms_documents = response.json()['entities']
            for entity in tms_documents:
                doc_info = {'title': entity['properties']['title'], 'extension': entity['properties']['extension']}
                tms_doc_info[entity['properties']['id']] = doc_info
        elif response.status_code == 204:
            logger.error('No documents to import!')
            return
        else:
            raise_error(response.json(), 'Error finding current documents in Lingotek Cloud')

        if import_all:
            ids_to_import = tms_doc_info.iterkeys()
        else:
            import_doc_info = {}
            for k, v in tms_doc_info.iteritems():
                import_doc_info[k] = v['title']
            ids_to_import = get_import_ids(import_doc_info)
        for curr_id in ids_to_import:
            self.import_document(curr_id, tms_doc_info[curr_id], force, path)

    def import_check(self, force, path, document_id, title):
        # local_ids = self.doc_manager.get_doc_ids()
        # if document_id in local_ids:
        #     if path:
        #         curr_entry = self.doc_manager.get_doc_by_prop('id', document_id)
        #         curr_path = os.path.join(self.path, curr_entry['file_name'])
        #         confirmation_msg = 'Would you like to overwrite the existing document? [y/N]:'
        #         if os.path.normpath(curr_path) != os.path.normpath(path):
        #             confirmation_msg = 'Would you like to overwrite the existing document and ' \
        #                                'its current saved path? [y/N]:'
        #         confirm = 'none'
        #         while confirm not in ['y', 'yes', 'n', 'no', '']:
        #             confirm = raw_input(confirmation_msg).lower()
        #         if not confirm or confirm in ['n', 'no']:
        #             logger.info('Skipped importing "{0}"'.format(title))
        #             return False
        #     else:
        #         logger.error('This document already exists somewhere locally. \
        #         Use the -p option to import it to a specified path or -f to force an update')
        #         return False
        # return True
        # check if document id is in local ids
        # if it is, document already exists -- check the path associated with local id
        # check current given path
        # if same, prompt for overwrite
        # if not same, prompt for overwrite for both path and file
        # move the file to the new path
        # if not just write to new file?
        if not path:
            path = self.path
        else:
            path = os.path.join(self.path, path.replace(self.path, ''))
        path_changed = False
        curr_entry = self.doc_manager.get_doc_by_prop('id', document_id)
        curr_path = os.path.join(self.path, curr_entry['file_name'])
        new_path = os.path.join(path, title)
        print curr_path, new_path
        if os.path.normpath(curr_path) != os.path.normpath(new_path):
            path_changed = True
        if not force:
            confirmation_msg = 'Would you like to overwrite the existing document? [y/N]:'
            if path_changed:
                confirmation_msg = 'Would you like to overwrite the existing document ' \
                                   'and its current saved path? [y/N]:'
            confirm = 'none'
            while confirm not in ['y', 'yes', 'n', 'no', '']:
                confirm = raw_input(confirmation_msg).lower()
            if not confirm or confirm in ['n', 'no']:
                logger.info('Skipped importing "{0}"'.format(title))
                path_changed = False

        return path_changed

    def import_document(self, document_id, document_info, force, path):
        local_ids = self.doc_manager.get_doc_ids()
        response = self.api.document_content(document_id, None, None)
        title, extension = os.path.splitext(document_info['title'])
        if not extension:
            extension = document_info['extension']
            extension = '.' + extension
        if extension and extension != '.none':
            title += extension
        if path:
            file_path = os.path.join(self.path, path, title)
        else:
            file_path = os.path.join(self.path, title)
        # file_path = os.path.join(os.getcwd(), title)  # import to current working directory
        logger.info('Importing "{0}" to {1}..'.format(title, file_path))
        # use status action to get locale info for importing
        try:
            locale_map = self.import_locale_info(document_id)
            locale_info = list(locale_map.iterkeys())
        except exceptions.RequestFailedError:
            locale_info = []

        changed_path = False
        if document_id in local_ids:
            changed_path = self.import_check(force, path, document_id, title)

            # if not force:
            #     self.import_check(path, document_id, title)
            # if document_id in local_ids:
            #     confirm = 'none'
            #     while confirm not in ['y', 'yes', 'n', 'no', '']:
            #         confirm = raw_input('Would you like to overwrite the existing document? [y/N]:').lower()
            #     if not confirm or confirm in ['n', 'no']:
            #         logger.info('Skipped importing "{0}"'.format(title))
            #         return
            # else:
            #     if self.doc_manager.get_doc_by_prop('file_name', file_path.replace(self.path, '')):
            #         # change file_path
            #         file_path = self.get_new_name(title, os.getcwd())
            #         orig_title = title
            #         title = os.path.basename(os.path.normpath(file_path))
            #         logger.warning(
            #             'Imported "{0}" as "{1}" because "{0}" already exists locally'.format(orig_title, title))
        # logger.info('Imported "{0}"'.format(title))
        if changed_path:
            self.delete_local(title, document_id, 'Moved local file {0}'.format(title))

        with open(file_path, 'wb') as fh:
            for chunk in response.iter_content(1024):
                fh.write(chunk)

        relative_path = file_path.replace(self.path, '')
        if document_id not in local_ids:
            self._add_document(relative_path, title, document_id)
            self.doc_manager.update_document('locales', locale_info, document_id)
        else:
            # update the document's path
            self.doc_manager.update_document('file_name', relative_path, document_id)



from tests.test_actions import *
from ltk.actions import Action
from io import BytesIO
import sys
import unittest

class TestList(unittest.TestCase):
    def setUp(self):
        create_config()
        self.action = Action(os.getcwd())

    def tearDown(self):
        cleanup()

    def test_list_doc(self):
        files = ['sample.txt', 'sample1.txt', 'sample2.txt']
        file_paths = []
        for fn in files:
            file_paths.append(create_txt_file(fn))
        self.action.add_action(None, ['sample*.txt'])
        doc_ids = self.action.doc_manager.get_doc_ids()
        try:
            out = BytesIO()
            sys.stdout = out
            self.action.list_ids_action()
            info = out.getvalue()
            for doc_id in doc_ids:
                assert doc_id in info
        finally:
            sys.stdout = sys.__stdout__
        for fn in files:
            self.action.delete_action(fn)
        self.action.clean_action(True, False, None)

    def test_list_no_docs(self):
        try:
            out = BytesIO()
            sys.stdout = out
            self.action.list_ids_action()
            info = out.getvalue()
            assert 'no documents' in info
        finally:
            sys.stdout = sys.__stdout__

    def test_list_workflow(self):
        try:
            out = BytesIO()
            sys.stdout = out
            self.action.list_workflow_action()
            info = out.getvalue()
            assert 'workflows' in info
            assert 'c675bd20-0688-11e2-892e-0800200c9a66' in info
            assert 'Machine Translation' in info
        finally:
            sys.stdout = sys.__stdout__

    def test_list_locale(self):
        try:
            out = BytesIO()
            sys.stdout = out
            self.action.list_locale_action()
            info = out.getvalue()
            assert 'ar_AE (Arabic, United Arab Emirates)' in info
            assert 'zh_TW (Chinese, Taiwan)' in info
        finally:
            sys.stdout = sys.__stdout__

    def test_list_format(self):
        try:
            out = BytesIO()
            sys.stdout = out
            self.action.list_format_action()
            info = out.getvalue()
            assert info.startswith('Formats Lingotek supports')
            assert 'CSV' in info
            assert 'XML_OKAPI' in info
        finally:
            sys.stdout = sys.__stdout__

    def test_list_filters_default(self):
        pass

    def test_list_filters_custom(self):
        # create custom filters
        # list
        # check
        pass



# the uri for the api calls
API_URI = {
    'community': '/api/community',
    'project': '/api/project',
    'project_id': '/api/project/%(project_id)s',
    'project_translation': '/api/project/%(project_id)s/translation',
    'project_translation_locale': '/api/project/%(project_id)s/translation/%(locale)s',
    'project_status': '/api/project/%(project_id)s/status',
    'document': '/api/document',
    'document_id': '/api/document/%(document_id)s',
    'document_translation': '/api/document/%(document_id)s/translation',
    'document_status': '/api/document/%(document_id)s/status',
    'document_content': '/api/document/%(document_id)s/content',
    'document_translation_locale': '/api/document/%(document_id)s/translation/%(locale)s',  # gets the status of target
    'workflow': '/api/workflow',
    'filter': '/api/filter'
}



from tests.test_actions import *
from ltk.actions import Action

import unittest

class TestImport(unittest.TestCase):
    def setUp(self):
        create_config()
        self.action = Action(os.getcwd())

    def tearDown(self):
        cleanup()

    def test_import_all(self):
        pass

    def test_import_locale(self):
        # test importing a document that already has a locale
        pass

    def test_import_no_locale(self):
        pass



from tests.test_actions import *
from ltk.actions import Action

import unittest

class TestRequest(unittest.TestCase):
    def setUp(self):
        create_config()
        self.action = Action(os.getcwd())

    def tearDown(self):
        cleanup()

    def test_request_one_locale_doc(self):
        pass

    def test_request_mult_locale_doc(self):
        pass

    def test_request_one_locale_proj(self):
        pass

    def test_request_mult_locale_proj(self):
        pass

    def test_delete_locale(self):
        pass



import click
import actions
import os
from exceptions import UninitializedError, ResourceNotFound, RequestFailedError, AlreadyExistsError
from constants import LOG_FN, CONF_DIR
import logging
from logger import logger, API_LOG_LEVEL, API_RESPONSE_LOG_LEVEL, CustomFormatter
import sys
from ltk import __version__
from watch import WatchAction
from import_action import ImportAction

def abort_if_false(ctx, param, value):
    if not value:
        ctx.abort()

def init_logger(path):
    """
    Initializes logger based on path
    """
    logger.setLevel(logging.DEBUG)
    if not path:
        file_handler = logging.FileHandler(LOG_FN)
    else:
        try:
            file_handler = logging.FileHandler(os.path.join(path, CONF_DIR, LOG_FN))
        except IOError:
            # todo error check when running init without existing conf dir
            os.mkdir(os.path.join(path, CONF_DIR))
            file_handler = logging.FileHandler(os.path.join(path, CONF_DIR, LOG_FN))
    console_handler = logging.StreamHandler(sys.stdout)
    file_handler.setLevel(API_LOG_LEVEL)
    file_handler.setFormatter(logging.Formatter('%(asctime)s  %(levelname)s: %(message)s'))
    if quiet:
        console_handler.setLevel(logging.WARNING)
    elif verbosity:
        if verbosity > 1:
            console_handler.setLevel(API_RESPONSE_LOG_LEVEL)
        else:
            console_handler.setLevel(API_LOG_LEVEL)
    else:
        console_handler.setLevel(logging.INFO)
    custom_formatter = CustomFormatter()
    console_handler.setFormatter(custom_formatter)
    logger.addHandler(file_handler)
    logger.addHandler(console_handler)

def print_log(error):
    """
    prints the error before logger is initialized
    """
    if not len(logger.handlers):
        print 'Error: {0}'.format(error)
        sys.exit()
    return

@click.group()
@click.version_option(version=__version__, message='%(prog)s version %(version)s (Lingotek Client)')
@click.option('-q', 'is_quiet', flag_value=True, help='will only show warnings')
@click.option('-v', 'verbosity_lvl', count=True, help='show API calls. -vv for API responses.')
def ltk(is_quiet, verbosity_lvl):
    global quiet, verbosity
    quiet = is_quiet
    verbosity = verbosity_lvl


@ltk.command()
@click.option('--access_token', help='your access token')
@click.option('--host', type=click.Choice(['myaccount.lingotek.com', 'cms.lingotek.com']), default='cms.lingotek.com',
              help='environment: myaccount for production, cms for sandbox; default is sandbox')
# @click.option('--host', help='host')
@click.option('--path', type=click.Path(exists=True),
              help='the path to the project directory to be initialized, defaults to current directory')
@click.option('-n', '--project_name', help='the preferred project name, defaults to current directory name')
@click.option('-w', '--workflow_id', default='c675bd20-0688-11e2-892e-0800200c9a66',
              help='the id of the workflow to use for this project, defaults to machine translate only.')
@click.option('-l', '--locale', default='en_US', help='the default source locale for the project; defaults to en_US')
@click.option('-d', '--delete', flag_value=True,  # expose_value=False, callback=abort_if_false,
              # prompt='Are you sure you want to delete the current project remotely and re-initialize? '
              #        'Use the -c flag if you only want to change the project.',
              help='delete the current project remotely and re-initialize')
# todo add a 'change' option so don't delete remote project
# @click.option('-c', '--change', flag_value=True, help='Change the Lingotek project. ')
@click.option('--reset', flag_value=True, help='re-authorize and reset any stored access tokens')
def init(host, access_token, path, project_name, workflow_id, locale, delete, reset):
    """ connects a local project to Lingotek """
    try:
        host = 'https://' + host
        if not path:
            path = os.getcwd()
        if not project_name:
            project_name = os.path.basename(os.path.normpath(path))
        init_logger(path)
        actions.init_action(host, access_token, path, project_name, workflow_id, locale, delete, reset)
    except (ResourceNotFound, RequestFailedError) as e:
        print_log(e)
        logger.error(e)
        return

@ltk.command()
@click.option('-l', '--locale', help='change the default source locale for project')
@click.option('-w', '--workflow_id', help='change the default workflow id for project')
@click.option('-d', '--download_folder', type=click.Path(exists=True),
              help='specify a folder for where downloaded translations should go')
@click.option('-f', '--watch_folder', type=click.Path(exists=True),
              help='specify a folder to watch when running ltk watch, defaults to project root')
@click.option('-t', '--target_locales', multiple=True,
              help='specify target locales that documents in watch_folder should be assigned, may either specify '
                   'with multiple -t flags (ex: -t locale -t locale) or give a list separated by commas and no spaces '
                   '(ex: -t locale,locale)')
def config(locale, workflow_id, download_folder, watch_folder, target_locales):
    """ view or change local configuration """
    try:
        action = actions.Action(os.getcwd())
        init_logger(action.path)
        action.config_action(locale, workflow_id, download_folder, watch_folder, target_locales)
    except (UninitializedError, RequestFailedError) as e:
        print_log(e)
        logger.error(e)
        return

@ltk.command(short_help="adds content, could be one or multiple files specified by Unix shell pattern")
@click.argument('file_names', required=True, nargs=-1)
@click.option('-l', '--locale', help='if source locale is different from the default configuration')
@click.option('-f', '--format',
              help='format of file; if not specified, will use extension to detect; defaults to plaintext')
@click.option('-s', '--srx', type=click.Path(exists=True), help='srx file')
@click.option('-si', '--srx_id', help='srx id')
@click.option('-i', '--its', type=click.Path(exists=True), help='its file')
@click.option('-ii', '--its_id', help='its id')
@click.option('-c', '--charset', help='file encoding')
@click.option('-ff', '--fprm', type=click.Path(exists=True), help='fprm file')
@click.option('-fi', '--fprm_id', help='fprm id')
@click.option('-fs', '--fprm_subfilter', type=click.Path(exists=True), help='fprm subfilter file')
@click.option('-fsi', '--fprm_subfilter_id', help='fprm subfilter id')
@click.option('-v', '--vault_id', help='save-to TM vault id')
@click.option('-e', '--external_url', help='source url')
def add(file_names, locale, **kwargs):
    """ adds content, could be one or multiple files specified by Unix shell pattern """
    try:
        action = actions.Action(os.getcwd())
        init_logger(action.path)
        action.add_action(locale, file_names, **kwargs)
    except (UninitializedError, RequestFailedError, ResourceNotFound, AlreadyExistsError) as e:
        print_log(e)
        logger.error(e)
        return

@ltk.command(short_help="sends updated content to Lingotek for documents that have been added")
def push():
    """ sends updated content to Lingotek for documents that have been added """
    try:
        action = actions.Action(os.getcwd())
        init_logger(action.path)
        action.push_action()
    except UninitializedError as e:
        print_log(e)
        logger.error(e)
        return


@ltk.command(short_help="add targets to document(s) to start translation, defaults to entire project")
@click.option('-n', '--doc_name', help='the name of the document, specify for one document')
@click.option('-d', '--delete', 'to_delete', flag_value=True, help='deletes a specified target locale')
@click.option('--due_date', help='the due date of the translation')
@click.option('-w', '--workflow', help='the workflow of the translation')
@click.argument('locales', required=True, nargs=-1)  # can have unlimited number of locales
def request(doc_name, locales, to_delete, due_date, workflow):
    """ add targets to document(s) to start translation, defaults to entire project """
    try:
        action = actions.Action(os.getcwd())
        init_logger(action.path)
        action.target_action(doc_name, locales, to_delete, due_date, workflow)
    except (UninitializedError, ResourceNotFound, RequestFailedError) as e:
        print_log(e)
        logger.error(e)
        return


# todo add a --all option to see all document ids once only show relative to cwd is implemented
@ltk.command(name='list')
@click.option('-d', '--documents', 'id_type', flag_value='document', help='list added documents')
@click.option('-w', '--workflows', 'id_type', flag_value='workflow', help='list available workflows')
@click.option('-l', '--locales', 'id_type', flag_value='locale', help='list supported locale codes')
@click.option('-f', '--formats', 'id_type', flag_value='format', help='list supported formats')
@click.option('--filters', 'id_type', flag_value='filter', help='list default and custom filters')
def list_ids(id_type):
    """ shows docs, workflows, locales, or formats """
    try:
        action = actions.Action(os.getcwd())
        init_logger(action.path)
        if id_type == 'workflow':
            action.list_workflow_action()
        elif id_type == 'locale':
            action.list_locale_action()
        elif id_type == 'format':
            action.list_format_action()
        elif id_type == 'filter':
            action.list_filter_action()
        else:
            action.list_ids_action()
    except (UninitializedError, RequestFailedError) as e:
        print_log(e)
        logger.error(e)
        return


@ltk.command(short_help="gets the status of a specific document or all documents")
@click.option('-n', '--doc_name', help='specific document name to get status of')
@click.option('-d', '--detailed', flag_value=True, help='detailed status of each locale for document')
def status(doc_name, detailed):
    """ gets the status of a specific document or all documents """
    try:
        action = actions.Action(os.getcwd())
        init_logger(action.path)
        action.status_action(detailed, doc_name)
    except (UninitializedError, ResourceNotFound) as e:
        print_log(e)
        logger.error(e)
        return


@ltk.command()
@click.option('-a', '--auto_format', flag_value=True, help='flag to auto apply formatting during download')
@click.argument('locale', required=True, nargs=1)
@click.argument('document_names', required=True, nargs=-1)
def download(auto_format, locale, document_names):
    """ downloads translated content of document(s) """
    try:
        action = actions.Action(os.getcwd())
        init_logger(action.path)
        for name in document_names:
            action.download_by_name(name, locale, auto_format)
    except (UninitializedError, ResourceNotFound, RequestFailedError) as e:
        print_log(e)
        logger.error(e)
        return


@ltk.command()
@click.option('-a', '--auto_format', flag_value=True, help='flag to auto apply formatting during download')
@click.argument('locales', nargs=-1)
def pull(auto_format, locales):
    """ pulls all translations for added documents """
    try:
        action = actions.Action(os.getcwd())
        init_logger(action.path)
        if locales:
            for locale in locales:
                action.pull_action(locale, auto_format)
        else:
            action.pull_action(None, auto_format)
    except UninitializedError as e:
        print_log(e)
        logger.error(e)
        return

@ltk.command(short_help="disassociate local doc(s) from Lingotek cloud and deletes remote copy")
@click.argument('document_names', required=True, nargs=-1)
@click.option('-f', '--force', flag_value=True, help='delete both local and remote files')
def rm(document_names, force):
    """
    disassociate local doc(s) from Lingotek cloud and deletes remote copy
    if remote copy should not be deleted, please use ltk clean
    """
    try:
        action = actions.Action(os.getcwd())
        init_logger(action.path)
        for name in document_names:
            action.delete_action(name, force)
    except (UninitializedError, ResourceNotFound, RequestFailedError) as e:
        print_log(e)
        logger.error(e)
        return

@ltk.command(name='import')
@click.option('-a', '--all', 'import_all', flag_value=True, help='import all documents from Lingotek Cloud')
@click.option('-f', '--force', flag_value=True, help='overwrites existing documents without prompt')
@click.option('-p', '--path', type=click.Path(exists=True), help='import documents to a specified path')
def import_command(import_all, force, path):
    """
    import documents from Lingotek cloud, automatically downloaded to project root
    """
    # todo import should show all documents
    # add a force option so can import all force -- overwrites all existing documents without prompting
    # check if doc id
        # if exist, prompt for overwrite
        # else automatically re-name
            # possibly have to patch title in Lingotek Cloud?
    try:
        # action = actions.Action(os.getcwd())
        action = ImportAction(os.getcwd())
        init_logger(action.path)
        action.import_action(import_all, force, path)
    except(UninitializedError, RequestFailedError) as e:
        print_log(e)
        logger.error(e)
        return

@ltk.command(short_help="cleans up the associations between local documents and documents in Lingotek")
@click.option('-a', '--all', 'dis_all', flag_value=True, help='removes all associations between local and remote')
@click.option('-n', '--doc_name', help='removes association of specified document name')
@click.option('-f', '--force', flag_value=True, help='deletes local documents that no longer exists in Lingotek')
def clean(force, dis_all, doc_name):
    """
    cleans up the associations between local documents and documents in Lingotek.
    by default, checks that local documents and remote documents line up.
    use different options for different use cases
    """
    try:
        action = actions.Action(os.getcwd())
        init_logger(action.path)
        action.clean_action(force, dis_all, doc_name)
    except (UninitializedError, RequestFailedError) as e:
        print_log(e)
        logger.error(e)
        return

@ltk.command(short_help="watches local and remote files")
@click.option('-p', '--path', type=click.Path(exists=True), help='specify a folder to watch, defaults to project path')
@click.option('--ignore', multiple=True, help='specify types of files to ignore')
@click.option('--auto', 'delimiter', help='automatically detects locale from the file name, specify locale delimiter')
def watch(path, ignore, delimiter):
    """
    Watches local files added or imported by ltk, and sends a PATCH when a document is changed.
    Also watches remote files, and automatically downloads finished translations.
    """
    try:
        action = WatchAction(os.getcwd())
        init_logger(action.path)
        action.watch_action(path, ignore, delimiter)
    except (UninitializedError, RequestFailedError) as e:
        print_log(e)
        logger.error(e)

if __name__ == '__main__':
    ltk()



from tests.test_actions import *
from ltk.actions import Action

import unittest

class TestClean(unittest.TestCase):
    def setUp(self):
        create_config()
        self.action = Action(os.getcwd())

    def tearDown(self):
        cleanup()

    def test_clean(self):
        pass

    def test_clean_force(self):
        # test that force deletes the file
        pass

    def test_disassociate(self):
        pass

    def test_clean_single(self):
        pass



import requests
from api_uri import API_URI
import utils
from exceptions import RequestFailedError
from logger import logger


class ApiCalls:
    def __init__(self, host, access_token):
        self.host = host
        self.headers = {'Authorization': 'bearer ' + access_token}
        # self.cert = ('lingotek.crt', 'lingotek.key')

    def list_communities(self):
        """ gets the communities that a user is in """
        uri = API_URI['community']
        payload = {'limit': 100}
        r = requests.get(self.host + uri, headers=self.headers, params=payload)
        log_api('GET', uri, r)
        return r

    def list_projects(self, community_id):
        """ gets the projects a user has """
        uri = API_URI['project']
        payload = {'community_id': community_id, 'limit': 100}
        r = requests.get(self.host + uri, headers=self.headers, params=payload)
        log_api('GET', uri, r)
        return r

    def add_project(self, project_name, community_id, workflow_id):
        """ adds a project """
        uri = API_URI['project']
        payload = {'title': project_name, 'community_id': community_id, 'workflow_id': workflow_id}
        r = requests.post(self.host + uri, headers=self.headers, data=payload)
        log_api('POST', uri, r)
        return r

    def patch_project(self, project_id, workflow_id):
        """ updates a project """
        uri = (API_URI['project_id'] % locals())
        payload = {'project_id': project_id}
        if workflow_id:
            payload['workflow_id'] = workflow_id
        r = requests.patch(self.host + uri, headers=self.headers, data=payload)
        log_api('PATCH', uri, r)
        return r

    def project_add_target(self, project_id, locale, due_date):
        """ adds a target to all documents within a project """
        uri = (API_URI['project_translation'] % locals())
        payload = {'id': project_id, 'locale_code': locale}
        if due_date:
            payload['due_date'] = due_date
        r = requests.post(self.host + uri, headers=self.headers, data=payload)
        log_api('POST', uri, r)
        return r

    def project_status(self, project_id):
        """ gets the status of a project """
        uri = (API_URI['project_status'] % locals())
        r = requests.get(self.host + uri, headers=self.headers)
        log_api('GET', uri, r)
        return r

    def project_delete_target(self, project_id, locale):
        uri = (API_URI['project_translation_locale'] % locals())
        r = requests.delete(self.host + uri, headers=self.headers)
        log_api('DELETE', uri, r)
        return r

    def delete_project(self, project_id):
        """ deletes a project """
        uri = (API_URI['project_id'] % locals())
        r = requests.delete(self.host + uri, headers=self.headers)
        log_api('DELETE', uri, r)
        return r

    def get_document(self, document_id):
        """ gets a document by id """
        uri = (API_URI['document_id'] % locals())
        r = requests.get(self.host + uri, headers=self.headers)
        log_api('GET', uri, r)
        return r

    def add_document(self, file_name, locale, project_id, title, **kwargs):
        """ adds a document """
        uri = API_URI['document']
        payload = {'locale_code': locale, 'project_id': project_id, 'title': title}
        for key, value in kwargs.iteritems():
            if kwargs[key]:
                payload[key] = value
        detected_format = utils.detect_format(file_name)
        if 'format' not in kwargs and detected_format != 'PLAINTEXT_OKAPI':
            payload['format'] = detected_format
        files = {'content': (file_name, open(file_name, 'rb'))}
        r = requests.post(self.host + uri, headers=self.headers, data=payload, files=files)
        log_api('POST', uri, r)
        return r

    def document_add_target(self, document_id, locale, workflow_id=None, due_date=None):
        """ adds a target to existing document, starts the workflow """
        uri = (API_URI['document_translation'] % locals())
        payload = {'locale_code': locale, 'id': document_id}
        if workflow_id:
            payload['workflow_id'] = workflow_id
        if due_date:
            payload['due_date'] = due_date
        r = requests.post(self.host + uri, headers=self.headers, data=payload)
        log_api('POST', uri, r)
        return r

    def list_documents(self, project_id):
        """ lists all documents a user has access to, could be filtered by project id """
        uri = API_URI['document']
        payload = {}
        if project_id:
            payload = {'project_id': project_id, 'limit': 1000}
        r = requests.get(self.host + uri, headers=self.headers, params=payload)
        log_api('GET', uri, r)
        return r

    def document_status(self, document_id):
        """ gets the status of a document """
        uri = (API_URI['document_status'] % locals())
        payload = {'document_id': document_id}
        r = requests.get(self.host + uri, headers=self.headers, params=payload)
        # logger.debug(r.url)
        log_api('GET', uri, r)
        return r

    def document_translation_status(self, document_id):
        """ gets the status of document translations """
        uri = (API_URI['document_translation'] % locals())
        r = requests.get(self.host + uri, headers=self.headers)
        log_api('GET', uri, r)
        return r

    def document_content(self, document_id, locale_code, auto_format):
        """ downloads the translated document """
        uri = (API_URI['document_content'] % locals())
        payload = {}
        if locale_code:
            payload['locale_code'] = locale_code
        if auto_format:
            payload['auto_format'] = auto_format
        r = requests.get(self.host + uri, headers=self.headers, params=payload, stream=True)
        log_api('GET', uri, r)
        return r

    def document_update(self, document_id, file_name=None, **kwargs):
        uri = (API_URI['document_id'] % locals())
        payload = {'id': document_id}
        for key, value in kwargs.iteritems():
            if kwargs[key]:
                payload[key] = value
        if file_name:
            files = {'content': (file_name, open(file_name, 'rb'))}
            r = requests.patch(self.host + uri, headers=self.headers, data=payload, files=files)
        else:
            r = requests.patch(self.host + uri, headers=self.headers, data=payload)
        log_api('PATCH', uri, r)
        return r

    def document_delete_target(self, document_id, locale):
        uri = (API_URI['document_translation_locale'] % locals())
        r = requests.delete(self.host + uri, headers=self.headers)
        log_api('DELETE', uri, r)
        return r

    def document_delete(self, document_id):
        uri = (API_URI['document_id'] % locals())
        r = requests.delete(self.host + uri, headers=self.headers)
        log_api('DELETE', uri, r)
        return r

    def list_workflows(self, community_id):
        uri = API_URI['workflow']
        payload = {'community_id': community_id}
        r = requests.get(self.host + uri, headers=self.headers, params=payload)
        log_api('GET', uri, r)
        return r

    def list_locales(self):
        uri = 'http://gmc.lingotek.com/v1/locales'
        r = requests.get(uri)
        return r

    def list_filters(self):
        uri = API_URI['filter']
        r = requests.get(self.host + uri, headers=self.headers)
        log_api('GET', uri, r)
        return r

    def get_project_info(self, community_id):
        response = self.list_projects(community_id)
        info = {}
        if response.status_code == 200:
            if int(response.json()['properties']['total']) == 0:
                return info
            entities = response.json()['entities']
            for entity in entities:
                info[entity['properties']['id']] = entity['properties']['title']
            return info
        elif response.status_code == 204:
            return info
        else:
            raise RequestFailedError("Unable to get existing projects")

    def get_communities_info(self):
        response = self.list_communities()
        if response.status_code != 200:
            raise RequestFailedError("Unable to get user's list of communities")
        entities = response.json()['entities']
        info = {}
        for entity in entities:
            info[entity['properties']['id']] = entity['properties']['title']
        return info


def log_api(method, uri, response):
    try:
        content_length = response.headers['content-length']
    except KeyError:
        content_length = 'No content-length header'
    log = '{method} {uri} {status} {length}'.format(method=method, uri=uri, status=response.status_code,
                                                    length=content_length)
    logger.api_call(log)
    try:
        import json
        response_info = json.dumps(response.json(), indent=4, separators=(',', ': '))
    except ValueError:
        response_info = 'No json response available'
    logger.api_response(response_info)



import os
from locales import default_locales
# from constants import APP_ID

class Enum(set):
    def __getattr__(self, name):
        if name in self:
            return name
        raise AttributeError

# def get_access_token(host, username, password):
#     auth_uri = host + '/auth/authorize.html'
#     auth_params = {'client_id': APP_ID, 'redirect_uri': '', 'response_type': 'token'}

# todo possibly put dictionary outside so not built with every function call
def detect_format(file_name, get_mapper=False):
    format_mapper = {
        '.csv': 'CSV',
        '.dita': 'DITA',
        '.ditamap': 'DITAMAP',
        '.docx': 'DOCX_OKAPI',
        '.dtd': 'DTD',
        '.xslx': 'EXCEL_OKAPI',  # xslx, xltx
        '.idml': 'IDML',
        '.properties': 'JAVA_PROPERTIES_OKAPI',
        '.json': 'JSON',
        '.pdf': 'PDF',
        '.txt': 'PLAINTEXT_OKAPI',
        '.po': 'PO',
        '.ppt': 'PPT_OKAPI',
        '.pptx': 'PPTX_OKAPI',
        '.resx': 'RESX',
        '.rtf': 'RTF_OKAPI',
        '.srt': 'SUBTITLE_RIP',
        '.tsv': 'TABLE',  # catkeys?
        '.ts': 'TS',
        '.xml': 'XML_OKAPI'
    }

    format_mapper.update(dict.fromkeys(['.dox', '.c', '.h', '.cpp'], 'DOXYGEN'))
    format_mapper.update(dict.fromkeys(['.html', '.htm'], 'HTML_OKAPI'))
    format_mapper.update(dict.fromkeys(['.odp', '.otp'], 'ODP'))
    format_mapper.update(dict.fromkeys(['.ods', '.ots'], 'ODS'))
    format_mapper.update(dict.fromkeys(['.odt', '.ott'], 'ODT'))
    format_mapper.update(dict.fromkeys(['.yaml', '.yml'], 'RAILS_YAML'))
    format_mapper.update(dict.fromkeys(['.xliff', '.xlf'], 'XLIFF_OKAPI'))
    if get_mapper:
        return format_mapper
    name, extension = os.path.splitext(file_name)
    return format_mapper.get(extension, 'PLAINTEXT_OKAPI')

def map_locale(locale):
    """
    maps incorrectly formatted locales to valid locales for use with Lingotek API
    :param locale: incorrectly formatted locale
    :return: valid locale
    """
    # import json
    # valid_locales = []
    # unsupported_locales = []
    # with open('data/language-default-locales.json') as json_file:
    #     default_locales = json.load(json_file)
    try:
        return default_locales[locale]
    except KeyError:
        return None
    # for locale in locales:
    #     try:
    #         valid_locales.append(default_locales[locale])
    #     except KeyError:
    #         unsupported_locales.append(locale)
    # return valid_locales, unsupported_locales



from logger import logger
from watchdog.events import FileSystemEventHandler

class WatchHandler(FileSystemEventHandler):
    def __init__(self):
        FileSystemEventHandler.__init__(self)

    def process(self, event):
        logger.info(event.event_type)

    def on_modified(self, event):
        self.process(event)

    def on_created(self, event):
        self.process(event)

    # on modified
    # on moved
    # on deleted



import socket
import sys

# from six.moves import BaseHTTPServer
# from six.moves import urllib
from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
import urlparse
import urllib
# import warnings
#
# with warnings.catch_warnings():
#     warnings.filterwarnings("ignore")
#     import webbrowser

class ClientRedirectServer(HTTPServer):
    """ A server to handle OAuth 2.0 redirects back to localhost.
    Waits for a single request and parses the query parameters
    into query_params and then stops serving.
    """
    query_params = {}

class ClientRedirectHandler(BaseHTTPRequestHandler, object):
    """ A handler for OAuth 2.0 redirects back to localhost.
    Waits for two requests and parses the access token
    into the servers query_params and then stops serving.
    """
    def do_GET(self):
        """ Handle a GET request.
            opens index.html and try to parse token
        """
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()
        self.wfile.write(
            b"<html><head><title>Authentication Status</title>")
        self.wfile.write(
            b'<script src="https://ajax.googleapis.com/ajax/libs/jquery/2.1.4/jquery.min.js"></script></head>')
        self.wfile.write(
            b"<link href='https://fonts.googleapis.com/css?family=Open+Sans' rel='stylesheet' type='text/css'>")
        self.wfile.write(
            b'<body style=\'font-size: 1.5em; padding: 50px;\'>')
        self.wfile.write(b'<p id="message" style="font-family: Open Sans, Arial; background-color: #eee; text-align: center; border: 1px solid #5cb85e; padding: 5px 20px 25px 20px;">Retrieving your access token...</p>')
        self.wfile.write(b'<script> \
            $(document).ready(function(){ \
                function getParamFromHash(url, param) { \
                    var re = new RegExp("#.*" + param + "=([^&]+)(&|$)"); \
                    var match = url.match(re); \
                    return(match ? match[1] : ""); \
                } \
                var self_url = window.location.href; \
                var token_info = self_url.split("#")[1]; \
                var params = { \
                    access_token: getParamFromHash(self_url, "access_token"), \
                    expires_in: getParamFromHash(self_url, "expires_in"), \
                    token_type: getParamFromHash(self_url, "token_type") \
                }; \
                console.log(params); \
                $.post("index.html", params).done(function(data) { \
                    $("#message").css("background-color","#dff0d9"); \
                    $("#message").html("<div><p style=\'font-weight: bold; color: darkgreen;\'>'
                         b'Your access token has been successfully stored!</p>'
                         b'<p style=\'color: #666; font-size: .8em\'>You may now close this browser window and return to the terminal.</p>'
                         b'<p style=\'color: #aaa; font-size: .5em\'>" + params["access_token"] + "</p></div>"); \
                }); \
            }); \
        </script>')
        self.wfile.write(b'</body></html>')

    def do_POST(self):
        """ Handle a POST request.
            Should only ever be sending self urlencoded so
        """
        length = int(self.headers['content-length'])
        post_vars = urlparse.parse_qsl(self.rfile.read(length))
        self.server.query_params = dict(post_vars)
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()
        self.wfile.write(b"<html><head><title>Authentication Response Status</title></head><body>")
        self.wfile.write(b"<p>Authentication has completed.</p>")
        self.wfile.write(b"</body></html>")


def run_oauth(host):
    r_host = 'localhost'  # host to redirect to
    r_ports = [9000, 9001, 9002]
    httpd = None
    server_started = False
    r_port = 0
    for port in r_ports:
        r_port = port
        try:
            httpd = ClientRedirectServer((r_host, port), ClientRedirectHandler)
        except socket.error:
            pass
        else:
            server_started = True
            break
    if not server_started:
        sys.exit('Unable to start a local webserver listening on port 9000, 9001, or 9002. '
                 'Please unblock one of these ports for authorizing with Lingotek. '
                 'This local webserver will stop serving after two requests and free the port up again.')
    oauth_callback = 'http://{0}:{1}/'.format(r_host, r_port)
    client_id = 'ab33b8b9-4c01-43bd-a209-b59f933e4fc4'
    response_type = 'token'
    payload = {'client_id': client_id, 'redirect_uri': oauth_callback, 'response_type': response_type}
    authorize_url = host + '/auth/authorize.html?' + urllib.urlencode(payload)
    import webbrowser
    webbrowser.open_new(authorize_url)
    print 'Your browser has been opened to visit: \n{0}\n'.format(authorize_url)
    print '--------------------------------------'
    httpd.handle_request()  # handle the GET redirect
    httpd.handle_request()  # handle the POST for token info
    print '--------------------------------------\n'
    if 'access_token' in httpd.query_params:
        print 'Access token has been successfully stored!'
        print '(If you haven\'t already, you may close your browser.)\n'
        init_token = httpd.query_params['access_token']
        token = init_token.split('&')[0]
        # store the token because apparently it doesn't expire..
        return token
    sys.exit('Something went wrong with the authentication request, please try again.')

# run_oauth('https://cms.lingotek.com')



import ctypes
from ltk.actions import Action
from logger import logger
from utils import map_locale
import time
import requests
import os
from watchdog.observers import Observer
from ltk.watchhandler import WatchHandler

# import Queue

# import threading

# class WatchThread:
#     def __init__(self, interval=5):
#         self.interval = interval
#
#         self.thread = threading.Thread(target=self.run, args=())
#         self.thread.daemon = True
#         self.thread.start()_on_create
#
#     def run(self):
#         while True:
#             time.sleep(self.interval)

# retry decorator to retry connections
def retry(logger, timeout=5, exec_type=None):
    if not exec_type:
        exec_type = [requests.exceptions.ConnectionError]

    def decorator(function):
        def wrapper(*args, **kwargs):
            while True:
                try:
                    return function(*args, **kwargs)
                except Exception as e:
                    if e.__class__ in exec_type:
                        # some logging error here
                        logger.error("Connection has timed out. Retrying..")
                        # sleep for some time then retry
                        time.sleep(timeout)
                    else:
                        raise e
        return wrapper
    return decorator

def is_hidden_file(file_path):
    # todo more robust checking for OSX files that doesn't start with '.'
    name = os.path.basename(os.path.abspath(file_path))
    return name.startswith('.') or has_hidden_attribute(file_path)

def has_hidden_attribute(file_path):
    """ Detects if a file has hidden attributes """
    try:
        attrs = ctypes.windll.kernel32.GetFileAttributesW(unicode(file_path))
        assert attrs != -1
        result = bool(attrs & 2)
    except (AttributeError, AssertionError):
        result = False
    return result

class WatchAction(Action):
    # def __init__(self, path, remote=False):
    def __init__(self, path):
        Action.__init__(self, path)
        self.observer = Observer()  # watchdog observer that will watch the files
        self.handler = WatchHandler()
        self.handler.on_modified = self._on_modified
        self.handler.on_created = self._on_created
        self.watch_queue = []  # not much slower than deque unless expecting 100+ items
        self.locale_delimiter = None
        self.ignore_ext = []  # file types to ignore as specified by the user
        # if remote:  # poll lingotek cloud periodically if this option enabled
        # self.remote_thread = threading.Thread(target=self.poll_remote(), args=())
        # self.remote_thread.daemon = True
        # self.remote_thread.start()

    def check_remote_doc_exist(self, fn, document_id=None):
        """ check if a document exists remotely """
        if not document_id:
            entry = self.doc_manager.get_doc_by_prop('file_name', fn)
            document_id = entry['id']
        response = self.api.get_document(document_id)
        if response.status_code != 200:
            return False
        return True

    def _on_modified(self, event):
        """ Notify Lingotek cloud when a previously added file is modified """
        db_entries = self.doc_manager.get_all_entries()
        in_db = False
        fn = ''
        # print event.src_path
        for entry in db_entries:
            # print entry['file_name']
            if event.src_path.endswith(entry['file_name']):
                fn = entry['file_name']
                in_db = True
        if not event.is_directory and in_db:
            # check that document is added in TMS before updating
            if self.check_remote_doc_exist(fn):
                # logger.info('Detected local content modified: {0}'.format(fn))
                # self.update_document_action(os.path.join(self.path, fn))
                # logger.info('Updating remote content: {0}'.format(fn))
                self.update_content(fn)

    def _on_created(self, event):
        # get path
        # add action
        file_path = event.src_path
        relative_path = file_path.replace(self.path, '')
        title = os.path.basename(os.path.normpath(file_path))
        curr_ext = os.path.splitext(file_path)[1]
        # return if the extension should be ignored or if the path is not a file
        if curr_ext in self.ignore_ext or not os.path.isfile(file_path):
            # logger.info("Detected a file with an extension in the ignore list, ignoring..")
            return
        if self.locale_delimiter:
            try:
                curr_locale = title.split(self.locale_delimiter)[1]
                fixed_locale = map_locale(curr_locale)
                if fixed_locale:
                    self.watch_locales.add(fixed_locale)
                else:
                    logger.warning('This document\'s detected locale: {0} is not supported.'.format(curr_locale))
            except IndexError:
                logger.warning('Cannot detect locales from file: {0}, not adding any locales'.format(title))
        # only add the document if it's not a hidden document and it's a new file
        if not is_hidden_file(file_path) and self.doc_manager.is_doc_new(relative_path):
            self.add_document(self.locale, file_path, title)
        elif self.doc_manager.is_doc_modified(relative_path, self.path):
            self.update_content(relative_path)
        document_id = self.doc_manager.get_doc_by_prop('name', title)['id']
        self.watch_add_target(title, document_id)
        # logger.info('Added new document {0}'.format(title))

    def watch_add_target(self, title, document_id):
        if self.check_remote_doc_exist(title, document_id):
            self.target_action(title, self.watch_locales, None, None, None, document_id)
            if document_id in self.watch_queue:
                self.watch_queue.pop(0)
        else:
            self.watch_queue.append(document_id)

    def process_queue(self):
        """do stuff with documents in queue (currently just add targets)"""
        # todo may want to process more than 1 item every "poll"..
        if self.watch_queue:
            self.watch_add_target(None, self.watch_queue.pop(0))

    def update_content(self, relative_path):
        logger.info('Detected local content modified: {0}'.format(relative_path))
        self.update_document_action(os.path.join(self.path, relative_path))
        logger.info('Updating remote content: {0}'.format(relative_path))

    @retry(logger)
    def poll_remote(self):
        """ poll lingotek servers to check if MT finished """
        # todo eventually: poll for other jobs (prefill, analyze, etc...)
        # print 'polling remote...'
        documents = self.doc_manager.get_all_entries()
        for doc in documents:
            doc_id = doc['id']
            if doc_id in self.watch_queue:
                # if doc id in queue, not imported yet
                continue
            locale_progress = self.import_locale_info(doc_id, True)
            try:
                downloaded = doc['downloaded']
            except KeyError:
                downloaded = []
                self.doc_manager.update_document('downloaded', downloaded, doc_id)
            for locale, progress in locale_progress.iteritems():
                if progress == 100 and locale not in downloaded:
                    logger.info('Translation completed ({0} - {1})'.format(doc_id,locale))
                    self.download_action(doc_id, locale, False)

    def watch_action(self, watch_path, ignore, delimiter):
        # print self.path
        if not watch_path and not self.watch_dir:
            watch_path = self.path
        elif watch_path:
            watch_path = os.path.join(self.path, watch_path)
        else:
            watch_path = self.watch_dir
        self.ignore_ext.extend(ignore)
        self.locale_delimiter = delimiter
        print "Watching for updates in: {0}".format(watch_path)
        self.observer.schedule(self.handler, path=watch_path, recursive=True)
        self.observer.start()
        try:
            while True:
                # print 'Watching....'
                self.poll_remote()
                self.process_queue()
                time.sleep(5)
        except KeyboardInterrupt:
            self.observer.stop()

        self.observer.join()



from tests.test_actions import *
from ltk import actions, exceptions
import unittest

class TestInitAction(unittest.TestCase):

    def test_uninitialized(self):
        self.assertRaises(exceptions.UninitializedError, actions.Action, os.getcwd())

    def test_init_host(self):
        create_config()
        action = actions.Action(os.getcwd())
        assert action.host
        cleanup()

    def test_init_access_token(self):
        create_config()
        action = actions.Action(os.getcwd())
        assert action.access_token
        cleanup()

    def test_init_project_id(self):
        create_config()
        action = actions.Action(os.getcwd())
        assert action.project_id
        cleanup()

    def test_init_path(self):
        create_config()
        action = actions.Action(os.getcwd())
        assert action.path
        cleanup()

    def test_init_community_id(self):
        create_config()
        action = actions.Action(os.getcwd())
        assert action.community_id
        cleanup()

    def test_init_api(self):
        create_config()
        action = actions.Action(os.getcwd())
        assert action.api
        cleanup()

    def test_init_doc_manager(self):
        create_config()
        action = actions.Action(os.getcwd())
        assert action.doc_manager
        cleanup()

# if __name__ == '__main__':
#     unittest.main()



class UninitializedError(Exception):
    """ Project has not been initialized """

class ResourceNotFound(Exception):
    """ Requested document is not found """

class AlreadyExistsError(Exception):
    """ Resource already exists """

class RequestFailedError(Exception):
    """ Request went wrong """




from setuptools import setup, find_packages
from ltk import __version__

setup(
    name='ltk',
    version=__version__,
    url='https://github.com/Lingotek/client',
    description='Command line tool that connects your content to the Lingotek Translation Network',
    author='Lingotek',
    author_email='integrations@lingotek.com',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'click',
        'requests[security]',
        'tinydb',
        'watchdog',
    ],
    extras_require={
        'security': ['pyOpenSSL', 'ndg-httpsclient', 'pyasn1'],
    },
    entry_points='''
        [console_scripts]
        ltk=ltk.commands:ltk
    ''',
)



from tests.test_actions import *
from ltk.actions import Action
import unittest

class TestPush(unittest.TestCase):
    def setUp(self):
        create_config()
        self.action = Action(os.getcwd())
        self.files = ['sample.txt', 'sample1.txt', 'sample2.txt']
        for fn in self.files:
            create_txt_file(fn)
        self.action.add_action(None, 'sample*.txt')
        self.doc_ids = self.action.doc_manager.get_doc_ids()
        for doc_id in self.doc_ids:
            assert poll_doc(self.action, doc_id)

    def tearDown(self):
        for curr_file in self.files:
            self.action.delete_action(curr_file)
        self.action.clean_action(True, False, None)
        cleanup()

    def test_push_1(self):
        # update one document
        # push
        # check results
        append_file(self.files[0])
        self.action.push_action()
        # todo check results

    def test_push_mult(self):
        # update mult docs
        # push
        # check
        pass

    def test_push_none(self):
        from io import BytesIO
        import sys

        try:
            out = BytesIO()
            sys.stdout = out
            self.action.push_action()
            info = out.getvalue()
            assert info == 'All documents up-to-date with Lingotek Cloud. '
        finally:
            sys.stdout = sys.__stdout__



from tinydb import TinyDB, where
import os
from constants import CONF_DIR, DB_FN

class DocumentManager:
    def __init__(self, path):
        db_file = os.path.join(path, CONF_DIR, DB_FN)
        self._db = TinyDB(db_file)

    def doc_exists(self, file_name, title):
        entries = self._db.search((where('file_name') == file_name) & (where('name') == title))
        if entries:
            return True
        else:
            return False

    def is_doc_new(self, file_name):
        file_name_exists = self._db.search(where('file_name') == file_name)
        if not file_name_exists:
            return True
        return False

    def is_doc_modified(self, file_name, path):
        entry = self._db.get(where('file_name') == file_name)
        full_path = os.path.join(path, file_name)
        last_modified = os.stat(full_path).st_mtime
        if entry and entry['added'] < last_modified and entry['last_mod'] < last_modified:
            return True
        return False

    def add_document(self, title, create_date, doc_id, sys_mtime, last_mod, file_name):
        entry = {'name': title, 'added': create_date, 'id': doc_id,
                 'sys_last_mod': sys_mtime, 'last_mod': last_mod, 'file_name': file_name,
                 'downloaded': []}
        self._db.insert(entry)

    def update_document(self, field, new_val, doc_id):
        if type(new_val) is list:
            self._db.update(_update_entry_list(field, new_val), where('id') == doc_id)
        else:
            if type(new_val) is set:
                new_val = list(new_val)
            self._db.update({field: new_val}, where('id') == doc_id)

    def get_doc_by_prop(self, prop, expected_value):
        """ get documents by the specified property """
        entry = self._db.get(where(prop) == expected_value)
        return entry

    def get_all_entries(self):
        return self._db.all()

    def get_doc_ids(self):
        """ returns all the ids of documents that user has added """
        doc_ids = []
        for entry in self._db.all():
            doc_ids.append(entry['id'])
        return doc_ids

    def remove_element(self, doc_id):
        self._db.remove(where('id') == doc_id)

    def clear_all(self):
        self._db.purge()

def _update_entry_list(field, new_val):
    """ updates a list in an entry """
    def transform(element):
        try:
            element[field]
        except KeyError:
            element[field] = []
        if new_val:
            # element[field].extend(new_val)
            element[field].extend([val.replace('-', '_') for val in new_val])
            element[field] = list(set(element[field]))
        else:
            element[field] = []

    return transform



from tests.test_actions import *
from ltk.actions import Action
from io import BytesIO
import sys
import unittest

class TestDownload(unittest.TestCase):
    def setUp(self):
        create_config()
        self.action = Action(os.getcwd())

    def tearDown(self):
        cleanup()

    def test_download_name_one(self):
        pass

    def test_download_name_mult(self):
        pass

    def test_pull(self):
        pass



from ltk.constants import CONF_DIR, CONF_FN
import os
import shutil
import time

def create_config():
    """
    create config folder and file to initialize without auth
    """
    conf_path = os.path.join(os.getcwd(), CONF_DIR)
    try:
        os.mkdir(conf_path)
    except OSError:
        pass
    new_config_file_path = os.path.join(conf_path, CONF_FN)
    config_file_path = os.path.join(os.getcwd(), 'config')
    shutil.copyfile(config_file_path, new_config_file_path)

def create_txt_file(file_name):
    file_path = os.path.join(os.getcwd(), file_name)
    with open(file_path, 'w') as txt_file:
        txt_file.write('This is a sample text file. ')
    return file_path

def append_file(file_name):
    file_path = os.path.join(os.getcwd(), file_name)
    with open(file_path, 'a') as txt_file:
        txt_file.write('Appended text. ')
    return file_path

def poll_doc(action, doc_id):
    """polls lingotek for the status of a document given id
        :returns True if document imported within 3min, else False
    """
    time_passed = 0
    while time_passed < 180:
        response = action.api.get_document(doc_id)
        if response.status_code == 200:
            return True
        time.sleep(1)
        time_passed += 1
    return False

def cleanup():
    conf_path = os.path.join(os.getcwd(), CONF_DIR)
    try:
        shutil.rmtree(conf_path)
    except OSError:
        print 'OSError in cleanup'

# test delete
# test import single
# test import all
# test clean



import ConfigParser
import os
import shutil
import fnmatch
import time
import exceptions
from apicalls import ApiCalls
from utils import detect_format
from managers import DocumentManager
from constants import CONF_DIR, CONF_FN, SYSTEM_FILE

from logger import logger


class Action:
    def __init__(self, path):
        self.host = ''
        self.access_token = ''
        self.project_id = ''
        self.project_name = ''
        self.path = path
        self.community_id = ''
        self.workflow_id = ''  # default workflow id; MT phase only
        self.locale = ''
        self.download_dir = None  # directory where downloaded translation will be stored
        self.watch_dir = None  # if specified, only watch this directory
        self.watch_locales = set()  # if specified, add these target locales to any files in the watch folder
        if not self._is_initialized():
            raise exceptions.UninitializedError("This project is not initialized. Please run init command.")
        self._initialize_self()
        self.api = ApiCalls(self.host, self.access_token)
        self.doc_manager = DocumentManager(self.path)

    def _is_initialized(self):
        actual_path = find_conf(self.path)
        if not actual_path:
            return False
        self.path = os.path.join(actual_path, '')
        if not is_initialized(self.path):
            return False
        return True

    def _initialize_self(self):
        config_file_name = os.path.join(self.path, CONF_DIR, CONF_FN)
        conf_parser = ConfigParser.ConfigParser()
        conf_parser.read(config_file_name)
        self.host = conf_parser.get('main', 'host')
        self.access_token = conf_parser.get('main', 'access_token')
        self.project_id = conf_parser.get('main', 'project_id')
        self.community_id = conf_parser.get('main', 'community_id')
        self.workflow_id = conf_parser.get('main', 'workflow_id')
        self.locale = conf_parser.get('main', 'default_locale')
        try:
            self.project_name = conf_parser.get('main', 'project_name')
            self.download_dir = conf_parser.get('main', 'download_folder')
            self.watch_dir = conf_parser.get('main', 'watch_folder')
            watch_locales = conf_parser.get('main', 'watch_locales')
            # todo watch_locales get overwritten the current way (reading from config)
            # since write to config then read from config.. intended?
            # self.watch_locales.extend(watch_locales.split(','))
            # self.watch_locales = list(set(self.watch_locales))
            self.watch_locales = set(watch_locales.split(','))
        except ConfigParser.NoOptionError:
            if not self.project_name:
                project_info = self.api.get_project_info(self.community_id)
                self.project_name = project_info[self.project_id]
                config_file_name, conf_parser = self.init_config_file()
                log_info = 'Updated project name'
                self.update_config_file('project_name', self.project_name, conf_parser, config_file_name, log_info)

    def _add_document(self, file_name, title, doc_id):
        """ adds a document to db """
        now = time.time()
        # doc_id = json['properties']['id']
        full_path = os.path.join(self.path, file_name)
        last_modified = os.stat(full_path).st_mtime
        self.doc_manager.add_document(title, now, doc_id, last_modified, now, file_name)

    def _update_document(self, file_name):
        """ updates a document in the db """
        now = time.time()
        file_path = os.path.join(self.path, file_name)
        # sys_last_modified = os.stat(file_name).st_mtime
        sys_last_modified = os.stat(file_path).st_mtime
        entry = self.doc_manager.get_doc_by_prop('file_name', file_name)
        doc_id = entry['id']
        self.doc_manager.update_document('last_mod', now, doc_id)
        self.doc_manager.update_document('sys_last_mod', sys_last_modified, doc_id)
        # whenever a document is updated, it should have new translations
        self.doc_manager.update_document('downloaded', [], doc_id)

    def init_config_file(self):
        config_file_name = os.path.join(self.path, CONF_DIR, CONF_FN)
        conf_parser = ConfigParser.ConfigParser()
        conf_parser.read(config_file_name)
        return config_file_name, conf_parser

    def update_config_file(self, option, value, conf_parser, config_file_name, log_info):
        conf_parser.set('main', option, value)
        with open(config_file_name, 'wb') as new_file:
            conf_parser.write(new_file)
        self._initialize_self()
        logger.info(log_info)

    def config_action(self, locale, workflow_id, download_folder, watch_folder, target_locales):
        config_file_name, conf_parser = self.init_config_file()
        if locale:
            log_info = 'Project default locale has been updated to {0}'.format(locale)
            self.update_config_file('default_locale', locale, conf_parser, config_file_name, log_info)
        if workflow_id:
            response = self.api.patch_project(self.project_id, workflow_id)
            if response.status_code != 204:
                raise_error(response.json(), 'Something went wrong trying to update workflow_id of project')
            log_info = 'Project default workflow has been updated to {0}'.format(workflow_id)
            self.update_config_file('workflow_id', workflow_id, conf_parser, config_file_name, log_info)
            conf_parser.set('main', 'workflow_id', workflow_id)
        if download_folder:
            download_path = os.path.join(self.path, download_folder)
            log_info = 'Set download folder to {0}'.format(download_folder)
            self.update_config_file('download_folder', download_path, conf_parser, config_file_name, log_info)
        if watch_folder:
            watch_path = os.path.join(self.path, watch_folder)
            log_info = 'Set watch folder to {0}'.format(watch_folder)
            self.update_config_file('watch_folder', watch_path, conf_parser, config_file_name, log_info)
        if target_locales:
            log_info = 'Added target locales: {} for watch folder'.format(
                ', '.join(target for target in target_locales))
            target_locales = ','.join(target for target in target_locales)
            self.update_config_file('watch_locales', target_locales, conf_parser, config_file_name, log_info)

        print 'host: {0}\naccess_token: {1}\nproject id: {2}\nproject name: {6}\ncommunity id: {3}\nworkflow id: {4}\n' \
              'locale: {5}\ndownloads folder: {7}\nwatch folder: {8}\nwatch target locales: {9}'.format(
            self.host, self.access_token, self.project_id, self.community_id, self.workflow_id, self.locale,
            self.project_name, self.download_dir, self.watch_dir, ', '.join(x for x in self.watch_locales))

    def add_document(self, locale, file_name, title, **kwargs):
        response = self.api.add_document(file_name, locale, self.project_id, title, **kwargs)
        if response.status_code != 202:
            raise_error(response.json(), "Failed to add document {0}".format(title), True)
        else:
            logger.info('Added document {0}'.format(title))
            relative_path = file_name.replace(self.path, '')
            self._add_document(relative_path, title, response.json()['properties']['id'])

    def add_action(self, locale, file_patterns, **kwargs):
        if not locale:
            locale = self.locale
        # format will be automatically detected by extension but may not be what user expects
        # use current working directory as root for files instead of project root
        matched_files = get_files(os.getcwd(), file_patterns)
        if not matched_files:
            raise exceptions.ResourceNotFound("Could not find the specified file/pattern")
        for file_name in matched_files:
            # title = os.path.basename(os.path.normpath(file_name)).split('.')[0]
            title = os.path.basename(os.path.normpath(file_name))
            relative_path = file_name.replace(self.path, '')
            if not self.doc_manager.is_doc_new(relative_path):
                if self.doc_manager.is_doc_modified(relative_path, self.path):
                    confirm = 'not confirmed'
                    while confirm != 'y' and confirm != 'Y' and confirm != 'N' and confirm != 'n' and confirm != '':
                        confirm = raw_input("This document already exists. Would you like to overwrite it? [y/N]: ")
                    # confirm if would like to overwrite existing document in Lingotek Cloud
                    if not confirm or confirm in ['n', 'N']:
                        continue
                    else:
                        logger.info('Overwriting document: {0} in Lingotek Cloud...'.format(title))
                        self.update_document_action(file_name, title, **kwargs)
                        continue
                else:
                    logger.error("This document has already been added: {0}".format(title))
                    continue
            # todo separate function somewhere around here maybe..
            self.add_document(locale, file_name, title, **kwargs)
            # response = self.api.add_document(file_name, locale, self.project_id, title, **kwargs)
            # if response.status_code != 202:
            #     raise_error(response.json(), "Failed to add document {0}".format(title), True)
            # else:
            #     logger.info('Added document {0}'.format(title))
            #     self._add_document(relative_path, title, response.json()['properties']['id'])

    def push_action(self):
        entries = self.doc_manager.get_all_entries()
        updated = False
        for entry in entries:
            if not self.doc_manager.is_doc_modified(entry['file_name'], self.path):
                continue
            response = self.api.document_update(entry['id'], os.path.join(self.path, entry['file_name']))
            if response.status_code != 202:
                raise_error(response.json(), "Failed to update document {0}".format(entry['name']), True)
            updated = True
            logger.info('Updated ' + entry['name'])
            self._update_document(entry['file_name'])
        if not updated:
            logger.info('All documents up-to-date with Lingotek Cloud. ')

    def update_document_action(self, file_name, title=None, **kwargs):
        print "updating document action.."
        print file_name
        relative_path = file_name.replace(self.path, '')
        entry = self.doc_manager.get_doc_by_prop('file_name', relative_path)
        try:
            document_id = entry['id']
        except TypeError:
            logger.error("Document name specified doesn't exist: {0}".format(title))
            return
        if title:
            response = self.api.document_update(document_id, file_name, title=title, **kwargs)
        else:
            response = self.api.document_update(document_id, file_name)
        if response.status_code != 202:
            raise_error(response.json(), "Failed to update document {0}".format(file_name), True)
        self._update_document(relative_path)

    def _target_action_db(self, to_delete, locales, document_id):
        if to_delete:
            curr_locales = self.doc_manager.get_doc_by_prop('id', document_id)['locales']
            updated_locales = set(curr_locales) - set(locales)
            self.doc_manager.update_document('locales', updated_locales, document_id)
        else:
            self.doc_manager.update_document('locales', list(locales), document_id)

    def target_action(self, document_name, locales, to_delete, due_date, workflow, document_id=None):
        change_db_entry = True
        if to_delete:
            expected_code = 204
            failure_message = 'Failed to delete target'
            info_message = 'Deleted locale'
        else:
            expected_code = 201
            failure_message = 'Failed to add target'
            info_message = 'Added target'
        if not document_name and not document_id:
            for locale in locales:
                response = self.api.project_add_target(self.project_id, locale, due_date) if not to_delete \
                    else self.api.project_delete_target(self.project_id, locale)
                if response.status_code != expected_code:
                    raise_error(response.json(), '{message} {locale} for project'.format(message=failure_message,
                                                                                         locale=locale), True)
                    change_db_entry = False
                    continue
                logger.info('{message} {locale} for project {project_name}'.format(message=info_message, locale=locale,
                                                                                   project_name=self.project_name))
            document_ids = self.doc_manager.get_doc_ids()
            if change_db_entry:
                for document_id in document_ids:
                    self._target_action_db(to_delete, locales, document_id)
        else:
            # todo: document name or file name? since file name will be relative to root
            # todo: clean this code up some
            if not document_id:
                entry = self.doc_manager.get_doc_by_prop('name', document_name)
                try:
                    document_id = entry['id']
                except TypeError:
                    logger.error('Document name specified doesn\'t exist: {0}'.format(document_name))
                    return
                    # raise exceptions.ResourceNotFound("Document name specified doesn't exist: {0}".format(document_name))
            if not document_name:
                entry = self.doc_manager.get_doc_by_prop('id', document_id)
                try:
                    document_name = entry['name']
                except TypeError:
                    logger.error('Document specified doesn\'t exist: {0}'.format(document_id))
                    return
            for locale in locales:
                response = self.api.document_add_target(document_id, locale, workflow, due_date) if not to_delete \
                    else self.api.document_delete_target(document_id, locale)
                if response.status_code != expected_code:
                    raise_error(response.json(), '{message} {locale} for document'.format(message=failure_message,
                                                                                          locale=locale), True)
                    change_db_entry = False
                    continue
                logger.info('{message} {locale} for document {name}'.format(message=info_message,
                                                                            locale=locale, name=document_name))
            if change_db_entry:
                self._target_action_db(to_delete, locales, document_id)

    def list_ids_action(self):
        """ lists ids of list_type specified """
        ids = []
        titles = []
        locales = []
        entries = self.doc_manager.get_all_entries()
        cwd = os.path.join(os.getcwd(), '')
        for entry in entries:
            if entry['file_name'].startswith(cwd.replace(self.path, '')):
                ids.append(entry['id'])
                relative_path = entry['file_name'].replace(cwd.replace(self.path, ''), '')
                titles.append(relative_path)
                try:
                    locales.append(entry['locales'])
                except KeyError:
                    locales.append(['none'])
        if not ids:
            print 'no documents'
            return
        print 'documents: id, title, locales'
        for i in range(len(ids)):
            info = '{id} \t {title} \t\t {locales}'.format(id=ids[i], title=titles[i],
                                                           locales=', '.join(locale for locale in locales[i]))
            print info

    def list_workflow_action(self):
        response = self.api.list_workflows(self.community_id)
        if response.status_code != 200:
            raise_error(response.json(), "Failed to list workflows")
        ids, titles = log_id_names(response.json())
        if not ids:
            print 'no workflows'
            return
        print 'workflows: id, title'
        for i in range(len(ids)):
            info = '{id} \t {title}'.format(id=ids[i], title=titles[i])
            print info

    def list_locale_action(self):
        locale_info = []
        response = self.api.list_locales()
        if response.status_code != 200:
            raise exceptions.RequestFailedError("Failed to get locale codes")
        locale_json = response.json()
        for entry in locale_json:
            locale_code = locale_json[entry]['locale']
            language = locale_json[entry]['language_name']
            country = locale_json[entry]['country_name']
            locale_info.append((locale_code, language, country))
        for locale in sorted(locale_info):
            if not len(locale[2]):  # Arabic
                print "{0} ({1})".format(locale[0], locale[1])
            else:
                print "{0} ({1}, {2})".format(locale[0], locale[1], locale[2])

    def list_format_action(self):
        format_mapper = detect_format(None, True)
        print 'Formats Lingotek supports:'
        for format_name in sorted(set(format_mapper.itervalues())):
            print format_name

    def list_filter_action(self):
        response = self.api.list_filters()
        if response.status_code != 200:
            raise_error(response.json(), 'Failed to get filters')
        filter_entities = response.json()['entities']
        print 'filters: id, title'
        for entry in filter_entities:
            properties = entry['properties']
            title = properties['title']
            filter_id = properties['id']
            print '{0}\t{1}\t'.format(filter_id, title)

    def status_action(self, detailed, document_name=None):
        if document_name is not None:
            entry = self.doc_manager.get_doc_by_prop('name', document_name)
            try:
                doc_ids = [entry['id']]
            except TypeError:
                raise exceptions.ResourceNotFound("Document name specified doesn't exist: {0}".format(document_name))
        else:
            doc_ids = self.doc_manager.get_doc_ids()
        # detailed_status = {}
        for doc_id in doc_ids:
            response = self.api.document_status(doc_id)
            if response.status_code != 200:
                raise_error(response.json(), "Failed to get status of document", True)
            else:
                title = response.json()['properties']['title']
                progress = response.json()['properties']['progress']
                print '{0}: {1}%'.format(title, progress)
                # print title + ': ' + str(progress) + '%'
                # for each doc id, also call /document/id/translation and get % of each locale
            if detailed:
                response = self.api.document_translation_status(doc_id)
                if response.status_code != 200:
                    raise_error(response.json(), 'Failed to get detailed status of document', True)
                try:
                    for entry in response.json()['entities']:
                        curr_locale = entry['properties']['locale_code']
                        curr_progress = entry['properties']['percent_complete']
                        print '\tlocale: {0} \t percent complete: {1}%'.format(curr_locale, curr_progress)
                        # detailed_status[doc_id] = (curr_locale, curr_progress)
                except KeyError:
                    continue
                    # return detailed_status

    def download_by_name(self, document_name, locale_code, auto_format):
        try:
            document_id = self.doc_manager.get_doc_by_prop('name', document_name)['id']
        except TypeError:
            logger.error("Document name specified doesn't exist: {0}".format(document_name))
            return
        self.download_action(document_id, locale_code, auto_format)

    def download_action(self, document_id, locale_code, auto_format):
        # if not os.path.isdir(os.path.join(self.path, TRANS_DIR)):
        #     os.mkdir(os.path.join(self.path, TRANS_DIR))
        response = self.api.document_content(document_id, locale_code, auto_format)
        if response.status_code == 200:
            entry = self.doc_manager.get_doc_by_prop('id', document_id)
            if not entry:
                doc_info = self.api.get_document(document_id)
                try:
                    file_title = doc_info.json()['properties']['title']
                    title, extension = os.path.splitext(file_title)
                    if not extension:
                        extension = doc_info.json()['properties']['extension']
                        extension = '.' + extension
                    if extension and extension != '.none':
                        title += extension
                except KeyError:
                    raise_error(doc_info.json(),
                                'Something went wrong trying to import document: {0}'.format(document_id), True)
                    return
                download_path = os.path.join(self.path, title)
                logger.info("Downloaded: {0}".format(title))
            elif not locale_code:
                logger.info("Tried to download an existing document, did nothing")
                return
            else:
                file_name = entry['file_name']
                download_dir = os.path.join(self.path, os.path.dirname(file_name))
                base_name = os.path.basename(os.path.normpath(file_name))
                name_parts = base_name.split('.')
                if len(name_parts) > 1:
                    name_parts.insert(-1, locale_code)
                    downloaded_name = '.'.join(part for part in name_parts)
                else:
                    downloaded_name = name_parts[0] + '.' + locale_code
                download_path = os.path.join(download_dir, downloaded_name)
                logger.info('Downloaded: {0} ({1} - {2})'.format(downloaded_name, name_parts[0], locale_code))
                self.doc_manager.update_document('downloaded', [locale_code], document_id)
            if self.download_dir:
                title = os.path.basename(os.path.normpath(download_path))
                download_path = os.path.join(self.download_dir, title)
            with open(download_path, 'wb') as fh:
                for chunk in response.iter_content(1024):
                    fh.write(chunk)
            return download_path
        else:
            raise_error(response.json(), 'Failed to download content for id: {0}'.format(document_id), True)

    def pull_action(self, locale_code, auto_format):
        if not locale_code:
            entries = self.doc_manager.get_all_entries()
            for entry in entries:
                try:
                    locales = entry['locales']
                    for locale in locales:
                        self.download_action(entry['id'], locale, auto_format)
                except KeyError:
                    self.download_action(entry['id'], None, auto_format)
        else:
            document_ids = self.doc_manager.get_doc_ids()
            for document_id in document_ids:
                self.download_action(document_id, locale_code, auto_format)

    def delete_action(self, document_name, force):
        try:
            entry = self.doc_manager.get_doc_by_prop('name', document_name)
            document_id = entry['id']
            print document_id
        except TypeError:
            logger.warn("Document name specified doesn't exist: {0}".format(document_name))
            return
            # raise exceptions.ResourceNotFound("Document name specified doesn't exist: {0}".format(document_name))
        response = self.api.document_delete(document_id)
        if response.status_code != 204:
            # raise_error(response.json(), "Failed to delete document {0}".format(document_name), True)
            logger.error("Failed to delete document {0}".format(document_name))
        else:
            logger.info("{0} has been deleted remotely.".format(document_name))
            if force:
                self.delete_local(document_name, document_id)
            self.doc_manager.remove_element(document_id)

    def get_new_name(self, file_name, curr_path):
        i = 1
        file_path = os.path.join(curr_path, file_name)
        name, extension = os.path.splitext(file_name)
        while os.path.isfile(file_path):
            new_name = '{name}({i}){ext}'.format(name=name, i=i, ext=extension)
            file_path = os.path.join(curr_path, new_name)
            i += 1
        return file_path

    def import_locale_info(self, document_id, poll=False):
        locale_progress = {}
        response = self.api.document_translation_status(document_id)
        if response.status_code != 200:
            if poll:
                return {}
            else:
                # raise_error(response.json(), 'Failed to get locale details of document', True)
                raise exceptions.RequestFailedError('Failed to get locale details of document')
        try:
            for entry in response.json()['entities']:
                curr_locale = entry['properties']['locale_code']
                curr_progress = int(entry['properties']['percent_complete'])
                curr_locale = curr_locale.replace('-', '_')
                locale_progress[curr_locale] = curr_progress
        except KeyError:
            pass
        return locale_progress

    # def _import(self, document_id, document_info, force, path):
    #     local_ids = self.doc_manager.get_doc_ids()
    #     response = self.api.document_content(document_id, None, None)
    #     title, extension = os.path.splitext(document_info['title'])
    #     if not extension:
    #         extension = document_info['extension']
    #         extension = '.' + extension
    #     if extension and extension != '.none':
    #         title += extension
    #     if path:
    #         file_path = os.path.join(self.path, path, title)
    #     else:
    #         file_path = os.path.join(self.path, title)
    #     # file_path = os.path.join(os.getcwd(), title)  # import to current working directory
    #     logger.info('Importing "{0}"'.format(title))
    #     # use status action to get locale info for importing
    #     locale_info = self.import_locale_info(document_id)
    #     # todo document id may be in local_ids but not in same area.. check that their file path is diff,
    #     # and ask if want to move -- if not move, is it a copy?
    #     if not force:
    #         if document_id in local_ids:
    #             confirm = 'none'
    #             while confirm not in ['y', 'yes', 'n', 'no', '']:
    #                 confirm = raw_input('Would you like to overwrite the existing document? [y/N]:').lower()
    #             if not confirm or confirm in ['n', 'no']:
    #                 logger.info('Skipped importing "{0}"'.format(title))
    #                 return
    #         else:
    #             if self.doc_manager.get_doc_by_prop('file_name', file_path.replace(self.path, '')):
    #                 # change file_path
    #                 file_path = self.get_new_name(title, os.getcwd())
    #                 orig_title = title
    #                 title = os.path.basename(os.path.normpath(file_path))
    #                 logger.warning(
    #                     'Imported "{0}" as "{1}" because "{0}" already exists locally'.format(orig_title, title))
    #     # logger.info('Imported "{0}"'.format(title))
    #     with open(file_path, 'wb') as fh:
    #         for chunk in response.iter_content(1024):
    #             fh.write(chunk)
    #     if document_id not in local_ids:
    #         relative_path = file_path.replace(self.path, '')
    #         self._add_document(relative_path, title, document_id)
    #         self.doc_manager.update_document('locales', list(locale_info.iterkeys()), document_id)

    # def import_action(self, import_all, force, path):
    #     response = self.api.list_documents(self.project_id)
    #     tms_doc_info = {}
    #     if response.status_code == 200:
    #         tms_documents = response.json()['entities']
    #         for entity in tms_documents:
    #             doc_info = {'title': entity['properties']['title'], 'extension': entity['properties']['extension']}
    #             tms_doc_info[entity['properties']['id']] = doc_info
    #     elif response.status_code == 204:
    #         logger.error('No documents to import!')
    #         return
    #     else:
    #         raise_error(response.json(), 'Error finding current documents in Lingotek Cloud')
    #
    #     if import_all:
    #         ids_to_import = tms_doc_info.iterkeys()
    #     else:
    #         import_doc_info = {}
    #         for k, v in tms_doc_info.iteritems():
    #             import_doc_info[k] = v['title']
    #         ids_to_import = get_import_ids(import_doc_info)
    #     for curr_id in ids_to_import:
    #         self._import(curr_id, tms_doc_info[curr_id], force, path)

    def clean_action(self, force, dis_all, document_name):
        if dis_all:
            # disassociate everything
            self.doc_manager.clear_all()
            return

        if document_name:
            try:
                entry = self.doc_manager.get_doc_by_prop('name', document_name)
                document_id = entry['id']
                self.doc_manager.remove_element(document_id)
            except TypeError:
                logger.warn("Document name specified doesn't exist: {0}".format(document_name))
            return

        response = self.api.list_documents(self.project_id)
        local_ids = self.doc_manager.get_doc_ids()
        tms_doc_ids = []
        if response.status_code == 200:
            tms_documents = response.json()['entities']
            for entity in tms_documents:
                tms_doc_ids.append(entity['properties']['id'])
        elif response.status_code == 204:
            pass
        else:
            raise_error(response.json(), 'Error trying to list documents in TMS for cleaning')
        locals_to_delete = [x for x in local_ids if x not in tms_doc_ids]

        # check local files
        db_entries = self.doc_manager.get_all_entries()
        for entry in db_entries:
            # if local file doesn't exist, remove entry
            if not os.path.isfile(os.path.join(self.path, entry['file_name'])):
                locals_to_delete.append(entry['id'])

        # remove entry for local doc -- possibly delete local file too?
        if locals_to_delete:
            for curr_id in locals_to_delete:
                removed_title = self.doc_manager.get_doc_by_prop('id', curr_id)['name']
                # todo somehow this line^ doc is null... after delete files remotely, then delete locally
                if force:
                    self.delete_local(removed_title, curr_id)
                self.doc_manager.remove_element(curr_id)
                logger.info('Removing association for document {0}'.format(removed_title))
        else:
            logger.info('Local documents already up-to-date with Lingotek cloud')
            return
        logger.info('Cleaned up associations between local documents and Lingotek cloud')

    def delete_local(self, title, document_id, message=None):
        message = 'Removed local file {0}'.format(title) if not message else message
        file_name = self.doc_manager.get_doc_by_prop('id', document_id)['file_name']
        try:
            os.remove(os.path.join(self.path, file_name))
            logger.info(message)
        except OSError:
            logger.info('Something went wrong trying to delete the local file.')


def raise_error(json, error_message, is_warning=False):
    try:
        error = json['messages'][0]
        # Sometimes api returns vague errors like 'Unknown error'
        if error == 'Unknown error':
            error = error_message
        if not is_warning:
            raise exceptions.RequestFailedError(error)
        # warnings.warn(error)
        logger.error(error)
    except (AttributeError, IndexError):
        if not is_warning:
            raise exceptions.RequestFailedError(error_message)
        # warnings.warn(error_message)
        logger.error(error_message)


def is_initialized(project_path):
    ltk_path = os.path.join(project_path, CONF_DIR)
    if os.path.isdir(ltk_path) and os.path.isfile(os.path.join(ltk_path, CONF_FN)) and \
            os.stat(os.path.join(ltk_path, CONF_FN)).st_size:
        return True
    return False


def reinit(host, project_path, delete, reset):
    if is_initialized(project_path) and not reset:
        logger.warning('This project is already initialized!')
        if not delete:
            return False
        confirm = 'not confirmed'
        while confirm != 'y' and confirm != 'Y' and confirm != 'N' and confirm != 'n' and confirm != '':
            confirm = raw_input(
                "Are you sure you want to delete the current project? "
                "This will also delete the project in your community. [y/N]: ")
        # confirm if would like to delete existing folder
        if not confirm or confirm in ['n', 'N']:
            return False
        else:
            # delete the corresponding project online
            logger.info('Deleting old project folder and creating new one...')
            config_file_name = os.path.join(project_path, CONF_DIR, CONF_FN)
            if os.path.isfile(config_file_name):
                old_config = ConfigParser.ConfigParser()
                old_config.read(config_file_name)
                project_id = old_config.get('main', 'project_id')
                access_token = old_config.get('main', 'access_token')
                api = ApiCalls(host, access_token)
                response = api.delete_project(project_id)
                if response.status_code != 204 and response.status_code != 404:
                    try:
                        error = response.json()['messages'][0]
                        raise exceptions.RequestFailedError(error)
                    except (AttributeError, IndexError):
                        raise exceptions.RequestFailedError("Failed to delete and re-initialize project")
                # delete existing folder
                to_remove = os.path.join(project_path, CONF_DIR)
                shutil.rmtree(to_remove)
            else:
                raise exceptions.ResourceNotFound("Cannot find config file, please re-initialize project")
            return access_token
    return True


def choice_mapper(info):
    mapper = {}
    import operator

    sorted_info = sorted(info.iteritems(), key=operator.itemgetter(1))

    index = 0
    for entry in sorted_info:
        mapper[index] = {entry[0]: entry[1]}
        index += 1
    for k, v in mapper.iteritems():
        print '({0}) {1} ({2})'.format(k, v.itervalues().next(), v.iterkeys().next())
    return mapper


def get_import_ids(info):
    mapper = choice_mapper(info)
    chosen_indices = ['none-chosen']
    while not set(chosen_indices) <= set(mapper.iterkeys()):
        choice = raw_input('Which documents to import? (Separate indices by comma) ')
        try:
            chosen_indices = map(int, choice.split(','))
        except ValueError:
            print 'Some unexpected, non-integer value was included'
    return [mapper[index].iterkeys().next() for index in chosen_indices]


def display_choice(display_type, info):
    if display_type == 'community':
        input_prompt = 'Which community should this project belong to? '
    elif display_type == 'project':
        input_prompt = 'Which existing project should be used? '
    else:
        raise exceptions.ResourceNotFound("Cannot display info asked for")
    mapper = choice_mapper(info)
    choice = 'none-chosen'
    while choice not in mapper.iterkeys():
        choice = raw_input(input_prompt)
        try:
            choice = int(choice)
        except ValueError:
            print("That's not a valid option!")
    logger.info('Selected "{0}" {1}.'.format(mapper[choice].itervalues().next(), display_type))
    return mapper[choice].iterkeys().next(), mapper[choice].itervalues().next()


def check_global():
    # check for a global config file
    home_path = os.path.expanduser('~')
    sys_file = os.path.join(home_path, SYSTEM_FILE)
    if os.path.isfile(sys_file):
        # get the access token
        conf_parser = ConfigParser.ConfigParser()
        conf_parser.read(sys_file)
        return conf_parser.get('main', 'access_token')
    else:
        return None


def create_global(access_token):
    """
    create a .lingotek file in user's $HOME directory
    """
    # go to the home dir
    home_path = os.path.expanduser('~')
    file_name = os.path.join(home_path, SYSTEM_FILE)
    sys_file = open(file_name, 'w')

    config_parser = ConfigParser.ConfigParser()
    config_parser.add_section('main')
    config_parser.set('main', 'access_token', access_token)
    config_parser.write(sys_file)
    sys_file.close()


def init_action(host, access_token, project_path, folder_name, workflow_id, locale, delete, reset):
    # check if Lingotek directory already exists
    to_init = reinit(host, project_path, delete, reset)
    if not to_init:
        return
    elif to_init is not True:
        access_token = to_init

    ran_oauth = False
    if not access_token:
        access_token = check_global()
        if not access_token or reset:
            from auth import run_oauth

            access_token = run_oauth(host)
            ran_oauth = True

    if ran_oauth:
        # create or overwrite global file
        create_global(access_token)

    api = ApiCalls(host, access_token)
    # create a directory
    try:
        os.mkdir(os.path.join(project_path, CONF_DIR))
    except OSError:
        pass

    logger.info('Initializing project...')
    config_file_name = os.path.join(project_path, CONF_DIR, CONF_FN)
    # create the config file and add info
    config_file = open(config_file_name, 'w')

    config_parser = ConfigParser.ConfigParser()
    config_parser.add_section('main')
    config_parser.set('main', 'access_token', access_token)
    config_parser.set('main', 'host', host)
    # config_parser.set('main', 'root_path', project_path)
    config_parser.set('main', 'workflow_id', workflow_id)
    config_parser.set('main', 'default_locale', locale)
    # get community id
    community_info = api.get_communities_info()
    if len(community_info) == 0:
        raise exceptions.ResourceNotFound('You are not part of any communities in Lingotek Cloud')
    if len(community_info) > 1:
        community_id, community_name = display_choice('community', community_info)
    else:
        community_id = community_info.iterkeys().next()
    config_parser.set('main', 'community_id', community_id)

    response = api.list_projects(community_id)
    if response.status_code != 200:
        raise_error(response.json(), 'Something went wrong trying to find projects in your community')
    project_info = api.get_project_info(community_id)
    if len(project_info) > 0:
        confirm = 'none'
        while confirm != 'y' and confirm != 'Y' and confirm != 'N' and confirm != 'n' and confirm != '':
            confirm = raw_input('Would you like to use an existing Lingotek project? [y/N]:')
        if confirm and confirm in ['y', 'Y', 'yes', 'Yes']:
            project_id, project_name = display_choice('project', project_info)
            config_parser.set('main', 'project_id', project_id)
            config_parser.set('main', 'project_name', project_name)
            config_parser.write(config_file)
            config_file.close()
            return
    project_name = raw_input("Please enter a new Lingotek project name: %s" % folder_name + chr(8) * len(folder_name))
    if not project_name:
        project_name = folder_name
    response = api.add_project(project_name, community_id, workflow_id)
    if response.status_code != 201:
        raise_error(response.json(), 'Failed to add current project to Lingotek Cloud')
    project_id = response.json()['properties']['id']
    config_parser.set('main', 'project_id', project_id)
    config_parser.set('main', 'project_name', project_name)

    config_parser.write(config_file)
    config_file.close()


def find_conf(curr_path):
    """
    check if the conf folder exists in current directory's parent directories
    """
    if os.path.isdir(os.path.join(curr_path, CONF_DIR)):
        return curr_path
    elif curr_path == os.path.abspath(os.sep):
        return None
    else:
        return find_conf(os.path.abspath(os.path.join(curr_path, os.pardir)))


def get_files(root, patterns):
    """ gets all files matching pattern from root
        pattern supports any unix shell-style wildcards (not same as RE) """
    matched_files = []
    # print root
    for pattern in patterns:
        # check if pattern contains subdirectory
        subdir_pat, fn_pat = os.path.split(pattern)
        if not subdir_pat:
            for path, subdirs, files in os.walk(root):
                for fn in fnmatch.filter(files, pattern):
                    matched_files.append(os.path.join(path, fn))
        else:
            for path, subdirs, files in os.walk(root):
                # print os.path.split(path)
                # subdir = os.path.split(path)[1]  # get current subdir
                search_root = os.path.join(root, '')
                subdir = path.replace(search_root, '')
                # print subdir, subdir_pat
                if fnmatch.fnmatch(subdir, subdir_pat):
                    for fn in fnmatch.filter(files, fn_pat):
                        matched_files.append(os.path.join(path, fn))

    # for path, subdirs, files in os.walk(root):
    #     # matched_files = any(fnmatch.fnmatch(files, p) for p in patterns)
    #     for pattern in patterns:
    #         for name in fnmatch.filter(files, pattern):
    #             # print 'found without subdir'
    #             # print os.path.join(path, name)
    #             matched_files.append(os.path.join(path, name))
    # print matched_files
    return matched_files


def log_id_names(json):
    """
    logs the id and titles from a json object
    """
    ids = []
    titles = []
    for entity in json['entities']:
        ids.append(entity['properties']['id'])
        titles.append(entity['properties']['title'])
    return ids, titles



from ltk import actions, exceptions
import unittest
from io import BytesIO
import sys
from tests.test_actions import *

class TestStatusAction(unittest.TestCase):
    def setUp(self):
        create_config()
        self.action = actions.Action(os.getcwd())
        self.file_name = 'sample.txt'
        self.file_path = create_txt_file(self.file_name)
        self.action.add_action(None, [self.file_name])
        self.doc_id = self.action.doc_manager.get_doc_ids()[0]
        assert poll_doc(self.action, self.doc_id)
        self.targets = ['ja-JP', 'de-DE']

    def tearDown(self):
        # remove the created file
        self.action.delete_action(self.file_name)
        self.action.clean_action(True, False, None)
        cleanup()

    def test_status(self):
        # see that there is a status
        try:
            out = BytesIO()
            sys.stdout = out
            self.action.status_action(False, self.file_name)
            status = out.getvalue()
            assert status.startswith('Status of {0}'.format(self.file_name))
        finally:
            sys.stdout = sys.__stdout__

    def test_status_detailed(self):
        # see that there are targets
        # request translations
        self.action.target_action(self.file_name, self.targets, False, None, None)
        try:
            out = BytesIO()
            sys.stdout = out
            self.action.status_action(True, self.file_name)
            status = out.getvalue()
            assert 'Status of {0}'.format(self.file_name) in status
            for target in self.targets:
                assert 'locale: {0}'.format(target) in status
        finally:
            sys.stdout = sys.__stdout__

    def test_status_no_target(self):
        # when no targets have been added
        try:
            out = BytesIO()
            sys.stdout = out
            self.action.status_action(True, self.file_name)
            status = out.getvalue()
            # todo this test fails because the error comes from logger not stdout
            assert 'ERROR: No translations exist for document {0}'.format(self.doc_id) in status
        finally:
            sys.stdout = sys.__stdout__



import logging

# add a custom level to show API calls
API_LOG_LEVEL = 15
logging.addLevelName(API_LOG_LEVEL, 'API')

# add another custom level to show API responses
API_RESPONSE_LOG_LEVEL = 12
logging.addLevelName(API_RESPONSE_LOG_LEVEL, 'API Response')

def log_api(self, message, *args, **kwargs):
    if self.isEnabledFor(API_LOG_LEVEL):
        self._log(API_LOG_LEVEL, message, args, **kwargs)

def log_api_response(self, message, *args, **kwargs):
    if self.isEnabledFor(API_RESPONSE_LOG_LEVEL):
        self._log(API_RESPONSE_LOG_LEVEL, message, args, **kwargs)

logging.Logger.api_call = log_api
logging.Logger.api_response = log_api_response

logger = logging.getLogger('lib')

class CustomFormatter(logging.Formatter):
    default_format = '%(levelname)s: %(message)s'
    info_format = '%(message)s'

    def __init__(self, fmt="%(levelno)s: %(message)s"):
        logging.Formatter.__init__(self, fmt)

    def format(self, record):
        # Save the original format configured by the user
        # when the logger formatter was instantiated
        format_orig = self._fmt

        # Replace the original format with one customized by logging level
        if record.levelno == logging.INFO:
            self._fmt = CustomFormatter.info_format
        else:
            self._fmt = CustomFormatter.default_format

        # Call the original formatter class to do the grunt work
        result = logging.Formatter.format(self, record)

        # Restore the original format configured by the user
        self._fmt = format_orig

        return result
